/*
 * Created on Jan 28, 2004
 *
 * Copyright TimeSys Corp. 2003
 * All Rights Reserved
 */
package org.eclipse.cdt.toolchain;

import java.util.Arrays;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import org.eclipse.cdt.core.CCorePlugin;
import org.eclipse.cdt.core.ICDescriptor;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExtension;
import org.eclipse.core.runtime.ILog;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.w3c.dom.Element;

/**
 * Singleton.  Basically it queries all the IToolchainFactories and combines 
 * their results into a single list.
 * @author jeremiah.lott
 */
public class ToolchainManager {

	private static final ToolchainManager INSTANCE = new ToolchainManager();

	public static final String PLUGIN_ID = "org.eclipse.cdt.toolchain";
	public static final String FACTORY_EXTENSION = "ToolchainFactory";
	
	public static final String TOOLCHAIN_DATA_ID = "org.eclipse.cdt.toolchain";
	public static final String FACTORY_ID_ATTR = "factoryId";
	public static final String TOOLCHAIN_ID_ATTR = "toolchainId";
	
	private Map toolchainFactories = new TreeMap();
	
	private ToolchainManager() {
		loadFactories();
	}
	
	private void loadFactories() {
		IConfigurationElement[] configElements;
		IToolchainFactory factory;
		String id;
		ILog log = Platform.getPlugin(PLUGIN_ID).getLog();
		IExtension[] extensions = 
			Platform.getPluginRegistry().getExtensionPoint(PLUGIN_ID, 
				FACTORY_EXTENSION).getExtensions();
		for (int i = 0; i < extensions.length; i++) {
			configElements = extensions[i].getConfigurationElements();
			for (int j = 0; j < configElements.length; j++) {
				try {
					factory = (IToolchainFactory)
						configElements[j].createExecutableExtension("class");
					id = configElements[j].getAttribute("id");
					if (id == null || id.equals("")) {
						//assert configElements.length == 0;
						id = extensions[i].getUniqueIdentifier();
					}
					//assert id != null
					//assert !toolchainFactories.contains(id)
					toolchainFactories.put(id, factory);
				} catch (CoreException ex) {
					log.log(ex.getStatus());
				} catch (ClassCastException ex) {
					log.log(new Status(Status.ERROR, PLUGIN_ID, Status.OK, 
							ex.getLocalizedMessage(), ex));
				}
			}
		}
	}
	
	/**
	 * return the singleton instance of the toolchain manager.
	 */
	public static ToolchainManager getInstance() {
		return INSTANCE;
	}
	
	/**
	 * returns all toolchains from all available factories.
	 */
	public IToolchain[] getToolchains() {
		IToolchainFactory factory;
		List ret = new LinkedList();
		Iterator it = toolchainFactories.values().iterator();
		while (it.hasNext()) {
			factory = (IToolchainFactory)it.next();
			ret.addAll(Arrays.asList(factory.getToolchains()));
		}
		return (IToolchain[])ret.toArray(new IToolchain[ret.size()]);
	}
	
	/**
	 * returns all available toolchain factories.
	 */
	public IToolchainFactory[] getToolchainFactories() {
		return (IToolchainFactory[])toolchainFactories.values().
			toArray(new IToolchainFactory[toolchainFactories.size()]);
	}
	
	/**
	 * return the toolchain factory with the given id.
	 */
	public IToolchainFactory getToolchainFactory(String factoryId) {
		return (IToolchainFactory)toolchainFactories.get(factoryId);
	}
	
	/**
	 * equivalent to getToolchainFactory(factoryId).getToolchain(toolchainId),
	 * except if the toolchain factory does not exist it will return null 
	 * (instead of an NPE).
	 */
	public IToolchain getToolchain(String factoryId, String toolchainId) {
		IToolchainFactory factory = getToolchainFactory(factoryId);
		if (factory == null) {
			return null;
		} else {
			return factory.getToolchain(toolchainId);
		}
	}
	
	/**
	 * reads the factory id and toolchain id from the project and returns
	 * the corresponding toolchain.  Returns null if the factory does not
	 * exist, the toolchain does not exist or if the project does not 
	 * reference a toolchain.
	 * TODO: do we want to allow only one toolchain per project (probably not)
	 * TODO: this returns null for too many possibilities, may want to have
	 * special handling for each of the reasons it can return null.
	 */
	public IToolchain getToolchain(IProject project) throws CoreException {
		ICDescriptor desc = CCorePlugin.getDefault().
			getCProjectDescription(project);
		Element element = desc.getProjectData(TOOLCHAIN_DATA_ID);
		String toolchainFactoryId = element.getAttribute(FACTORY_ID_ATTR);
		String toolchainId = element.getAttribute(TOOLCHAIN_ID_ATTR);
		if ((toolchainFactoryId == null || toolchainFactoryId.equals("")) || 
			(toolchainId == null || toolchainId.equals(""))) {
			// assert toolchainFactoryId == null && toolchainId == null
			return null;
		}
		IToolchainFactory factory = getToolchainFactory(toolchainFactoryId);
		if (factory == null) {
			return null;
		}
		return factory.getToolchain(toolchainId);		
	}
	
}
